"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const bent_1 = (0, tslib_1.__importDefault)(require("bent"));
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const md5_1 = (0, tslib_1.__importDefault)(require("./md5"));
const constants_1 = require("../constants");
const DownloadFontFileError_1 = (0, tslib_1.__importDefault)(require("../DownloadFontFileError"));
const save_stream_to_file_1 = (0, tslib_1.__importDefault)(require("./save-stream-to-file"));
const getStream = (0, bent_1.default)();
function isFile(filePath) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        try {
            const fileStat = yield (0, fs_extra_1.stat)(filePath);
            return fileStat.isFile();
        }
        catch (_a) {
            return false;
        }
    });
}
/**
 * 下载字体源文件
 *
 * @param fontUrl 字体源文件下载链接
 */
function downloadFontFile(fontUrl) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const key = (0, md5_1.default)(fontUrl);
        const fontFilePath = (0, path_1.resolve)(constants_1.FONTS_DIR, key);
        if (yield isFile(fontFilePath)) {
            return fontFilePath;
        }
        yield (0, fs_extra_1.ensureDir)(constants_1.FONTS_DIR);
        const stream = (yield getStream(fontUrl));
        if (stream.statusCode < 200 || stream.statusCode >= 300) {
            throw new DownloadFontFileError_1.default(fontUrl);
        }
        yield (0, save_stream_to_file_1.default)(stream, fontFilePath);
        return fontFilePath;
    });
}
exports.default = downloadFontFile;
//# sourceMappingURL=download-font-file.js.map