"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const os_1 = require("os");
const create_page_1 = (0, tslib_1.__importDefault)(require("./create-page"));
const path_1 = require("path");
const promises_1 = require("fs/promises");
const uuid_1 = require("uuid");
const tasks = [];
const pageStats = new Array((0, os_1.cpus)().length).fill(0);
const pages = [];
function getPage() {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const index = Math.floor(Math.random() * (0, os_1.cpus)().length);
        let page = pages[index];
        if (!page) {
            page = (0, create_page_1.default)();
            pages[index] = page;
            page.catch((e) => {
                console.log('启动 chrome 失败');
                console.error(e);
                pages[index] = null;
            });
        }
        return page;
    });
}
function htmlToPdf(html, format, landscape, res) {
    tasks.unshift({ html, res, format, landscape });
    execTasks();
}
exports.default = htmlToPdf;
let ROOT_PATH = (0, os_1.tmpdir)();
function execTask(page, task) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const pdfFilePath = (0, path_1.resolve)(ROOT_PATH, `${(0, uuid_1.v4)()}.pdf`);
        try {
            const pageInstance = yield page;
            yield pageInstance.setContent(task.html);
            yield pageInstance.pdf({
                path: pdfFilePath,
                format: task.format,
                displayHeaderFooter: false,
                landscape: task.landscape,
            });
            task.res.sendFile(pdfFilePath, () => {
                (0, promises_1.unlink)(pdfFilePath);
            });
        }
        catch (e) {
            (0, promises_1.unlink)(pdfFilePath).then(null, () => { });
            task.res.status(500).end();
        }
    });
}
function execTasks() {
    for (let i = 0; i < pageStats.length; i++) {
        if (tasks.length === 0) {
            break;
        }
        if (pageStats[i] === 0) {
            const page = getPage();
            const task = tasks.shift();
            pageStats[i] = 1;
            let startTime = new Date().getTime();
            execTask(page, task).finally(() => {
                console.log('用时：', new Date().getTime() - startTime);
                pageStats[i] = 0;
                execTasks();
            });
        }
    }
}
setInterval(execTasks, 25);
//# sourceMappingURL=html-to-pdf.js.map