"use strict";
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const commander_1 = require("commander");
const express_1 = (0, tslib_1.__importDefault)(require("express"));
const cors_1 = (0, tslib_1.__importDefault)(require("cors"));
const get_font_name_1 = (0, tslib_1.__importDefault)(require("./helpers/get-font-name"));
const get_package_info_1 = (0, tslib_1.__importDefault)(require("./helpers/get-package-info"));
const get_sub_font_with_cache_1 = (0, tslib_1.__importDefault)(require("./helpers/get-sub-font-with-cache"));
const html_to_pdf_1 = (0, tslib_1.__importDefault)(require("./html-to-pdf"));
const create_page_1 = (0, tslib_1.__importDefault)(require("./html-to-pdf/create-page"));
const program = new commander_1.Command()
    .version((_b = (_a = (0, get_package_info_1.default)()) === null || _a === void 0 ? void 0 : _a.version) !== null && _b !== void 0 ? _b : '1.0.0')
    .description('动态字体服务')
    .option('-p, --port [port]', '指定服务端口号', '8088')
    .parse(process.argv);
const { port } = program.opts();
const app = (0, express_1.default)();
app.use((0, cors_1.default)());
const dynamicFontApi = (fontUrl, text, res, next) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    if (!fontUrl || !text) {
        res.sendStatus(400);
        return;
    }
    try {
        const buffer = yield (0, get_sub_font_with_cache_1.default)(fontUrl, text);
        const fontName = yield (0, get_font_name_1.default)(fontUrl);
        res.status(200);
        /**
         * 设置下载的文件类型
         */
        res.set('Content-Type', 'font/truetype');
        // 下载文件名称
        res.set('Content-Disposition', `inline;filename=${fontName}.ttf`);
        // 缓存 365 天
        res.set('Cache-Control', 'public, max-age=31536000');
        res.end(buffer);
    }
    catch (e) {
        next(e);
    }
});
app.use(express_1.default.json());
const DYNAMIC_FONT_PATH = '/apis/intellisense-form/dynamic-font';
const HTML_TO_PDF_PATH = '/apis/intellisense-form/dynamic-font/html-to-pdf';
const CHECK_BROWSER_PATH = '/apis/intellisense-form/dynamic-font/check-browser';
app.get(CHECK_BROWSER_PATH, (_, res) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    try {
        const page = yield (0, create_page_1.default)();
        page.setContent('这是一段文本');
        yield page.pdf();
        page.browser().close();
        res.json('browser is ok');
    }
    catch (_c) {
        res.json('browser is error');
    }
}));
app.post(HTML_TO_PDF_PATH, (req, res) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const { html, format, landscape } = req.body;
    (0, html_to_pdf_1.default)(html, format, landscape, res);
}));
app.get(DYNAMIC_FONT_PATH, (req, res, next) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const { fontUrl, text } = req.query;
    yield dynamicFontApi(fontUrl, text, res, next);
}));
app.post(DYNAMIC_FONT_PATH, (req, res, next) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
    const { fontUrl, text } = JSON.parse(req.body);
    yield dynamicFontApi(fontUrl, text, res, next);
}));
// 在其他 app.use() 和路由调用之后，最后定义错误处理中间件
app.use((err, _req, _res, next) => {
    console.error(err);
    next(err);
});
app.use((err, _, res, next) => {
    if (res.headersSent) {
        return next(err);
    }
    res.status(500);
    res.send(err.stack);
});
app.listen(port, () => {
    console.log(`动态字体服务服务已启动，端口号为：${port}`);
});
//# sourceMappingURL=index.js.map